@@ -4,7 +4,7 @@
  * Copyright 2005-2007 - Paolo Borelli and Paolo Maggi
  * Copyright 2007 - Steve Frécinaux
  * Copyright 2008 - Jesse van den Kieboom
- * Copyright 2014, 2016 - Sébastien Wilmet
+ * Copyright 2014, 2016, 2017 - Sébastien Wilmet
  *
  * Gtef is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the
@@ -26,13 +26,14 @@
 #include "gtef-file.h"
 #include "gtef-buffer-input-stream.h"
 #include "gtef-buffer.h"
+#include "gtef-encoding.h"
 #include "gtef-enum-types.h"
 
 /**
  * SECTION:file-saver
  * @Short_description: Save a GtefBuffer into a file
  * @Title: GtefFileSaver
- * @See_also: #GtefFile
+ * @See_also: #GtefFile, #GtefFileLoader
  *
  * A #GtefFileSaver object permits to save a #GtefBuffer into a #GFile.
  *
@@ -40,8 +41,9 @@
  * handling. If an error occurs, you can reconfigure the saver and relaunch the
  * operation with gtef_file_saver_save_async().
  *
- * #GtefFileSaver is a non-modified fork of #GtkSourceFileSaver. See the
- * description of #GtefFile for more background on why a fork was needed.
+ * #GtefFileSaver is a fork of #GtkSourceFileSaver, the code has been a little
+ * improved (but no major changes). See the description of #GtefFile for more
+ * background on why a fork was needed.
  */
 
 /* The code has been written initially in gedit (GeditDocumentSaver).
@@ -58,8 +60,6 @@
 
 #define WRITE_CHUNK_SIZE 8192
 
-#define QUERY_ATTRIBUTES G_FILE_ATTRIBUTE_TIME_MODIFIED
-
 enum
 {
 	PROP_0,
@@ -88,9 +88,9 @@ struct _GtefFileSaverPrivate
 
 	GFile *location;
 
-	const GtkSourceEncoding *encoding;
-	GtkSourceNewlineType newline_type;
-	GtkSourceCompressionType compression_type;
+	GtefEncoding *encoding;
+	GtefNewlineType newline_type;
+	GtefCompressionType compression_type;
 	GtefFileSaverFlags flags;
 
 	GTask *task;
@@ -99,9 +99,11 @@ struct _GtefFileSaverPrivate
 typedef struct _TaskData TaskData;
 struct _TaskData
 {
-	/* The output_stream contains the required converter(s) for the encoding
-	 * and the compression type.
-	 * The two streams cannot be spliced directly, because:
+	GFileOutputStream *file_output_stream;
+
+	/* The output_stream contains the file_output_stream, plus the required
+	 * converter(s) for the encoding and the compression type.
+	 * input_stream and output_stream cannot be spliced directly, because:
 	 * (1) We need to call the progress callback.
 	 * (2) Sync methods must be used for the input stream, and async
 	 *     methods for the output stream.
@@ -109,8 +111,6 @@ struct _TaskData
 	GtefBufferInputStream *input_stream;
 	GOutputStream *output_stream;
 
-	GFileInfo *info;
-
 	goffset total_size;
 	GFileProgressCallback progress_cb;
 	gpointer progress_cb_data;
@@ -151,9 +151,9 @@ task_data_free (gpointer data)
 		return;
 	}
 
+	g_clear_object (&task_data->file_output_stream);
 	g_clear_object (&task_data->input_stream);
 	g_clear_object (&task_data->output_stream);
-	g_clear_object (&task_data->info);
 	g_clear_error (&task_data->error);
 
 	if (task_data->progress_cb_notify != NULL)
@@ -286,6 +286,16 @@ gtef_file_saver_dispose (GObject *object)
 	G_OBJECT_CLASS (gtef_file_saver_parent_class)->dispose (object);
 }
 
+static void
+gtef_file_saver_finalize (GObject *object)
+{
+	GtefFileSaver *saver = GTEF_FILE_SAVER (object);
+
+	gtef_encoding_free (saver->priv->encoding);
+
+	G_OBJECT_CLASS (gtef_file_saver_parent_class)->finalize (object);
+}
+
 static void
 gtef_file_saver_constructed (GObject *object)
 {
@@ -293,9 +303,9 @@ gtef_file_saver_constructed (GObject *object)
 
 	if (saver->priv->file != NULL)
 	{
-		const GtkSourceEncoding *encoding;
-		GtkSourceNewlineType newline_type;
-		GtkSourceCompressionType compression_type;
+		const GtefEncoding *encoding;
+		GtefNewlineType newline_type;
+		GtefCompressionType compression_type;
 
 		encoding = gtef_file_get_encoding (saver->priv->file);
 		gtef_file_saver_set_encoding (saver, encoding);
@@ -331,6 +341,7 @@ gtef_file_saver_class_init (GtefFileSaverClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->dispose = gtef_file_saver_dispose;
+	object_class->finalize = gtef_file_saver_finalize;
 	object_class->set_property = gtef_file_saver_set_property;
 	object_class->get_property = gtef_file_saver_get_property;
 	object_class->constructed = gtef_file_saver_constructed;
@@ -401,7 +412,7 @@ gtef_file_saver_class_init (GtefFileSaverClass *klass)
 					 g_param_spec_boxed ("encoding",
 							     "Encoding",
 							     "",
-							     GTK_SOURCE_TYPE_ENCODING,
+							     GTEF_TYPE_ENCODING,
 							     G_PARAM_READWRITE |
 							     G_PARAM_CONSTRUCT |
 							     G_PARAM_STATIC_STRINGS));
@@ -419,7 +430,7 @@ gtef_file_saver_class_init (GtefFileSaverClass *klass)
 					                    "Newline type",
 							    "",
 					                    GTK_SOURCE_TYPE_NEWLINE_TYPE,
-					                    GTK_SOURCE_NEWLINE_TYPE_LF,
+					                    GTEF_NEWLINE_TYPE_LF,
 					                    G_PARAM_READWRITE |
 					                    G_PARAM_CONSTRUCT |
 							    G_PARAM_STATIC_STRINGS));
@@ -437,7 +448,7 @@ gtef_file_saver_class_init (GtefFileSaverClass *klass)
 					                    "Compression type",
 					                    "",
 					                    GTK_SOURCE_TYPE_COMPRESSION_TYPE,
-					                    GTK_SOURCE_COMPRESSION_TYPE_NONE,
+					                    GTEF_COMPRESSION_TYPE_NONE,
 					                    G_PARAM_READWRITE |
 					                    G_PARAM_CONSTRUCT |
 							    G_PARAM_STATIC_STRINGS));
@@ -541,38 +552,6 @@ cancel_output_stream (GTask *task)
  * END NOTE
  */
 
-static void
-query_info_cb (GObject      *source_object,
-	       GAsyncResult *result,
-	       gpointer      user_data)
-{
-	GFile *location = G_FILE (source_object);
-	GTask *task = G_TASK (user_data);
-	TaskData *task_data;
-	GError *error = NULL;
-
-	DEBUG ({
-	       g_print ("Finished query info on file\n");
-	});
-
-	task_data = g_task_get_task_data (task);
-
-	g_clear_object (&task_data->info);
-	task_data->info = g_file_query_info_finish (location, result, &error);
-
-	if (error != NULL)
-	{
-		DEBUG ({
-		       g_print ("Query info failed: %s\n", error->message);
-		});
-
-		g_task_return_error (task, error);
-		return;
-	}
-
-	g_task_return_boolean (task, TRUE);
-}
-
 static void
 close_output_stream_cb (GObject      *source_object,
 			GAsyncResult *result,
@@ -580,15 +559,12 @@ close_output_stream_cb (GObject      *source_object,
 {
 	GOutputStream *output_stream = G_OUTPUT_STREAM (source_object);
 	GTask *task = G_TASK (user_data);
-	GtefFileSaver *saver;
 	GError *error = NULL;
 
 	DEBUG ({
 	       g_print ("%s\n", G_STRFUNC);
 	});
 
-	saver = g_task_get_source_object (task);
-
 	g_output_stream_close_finish (output_stream, result, &error);
 
 	if (error != NULL)
@@ -601,21 +577,8 @@ close_output_stream_cb (GObject      *source_object,
 		return;
 	}
 
-	/* Get the file info: note we cannot use
-	 * g_file_output_stream_query_info_async() since it is not able to get
-	 * the modification time.
-	 */
-	DEBUG ({
-	       g_print ("Query info on file\n");
-	});
-
-	g_file_query_info_async (saver->priv->location,
-			         QUERY_ATTRIBUTES,
-			         G_FILE_QUERY_INFO_NONE,
-				 g_task_get_priority (task),
-				 g_task_get_cancellable (task),
-			         query_info_cb,
-			         task);
+	/* Finished! */
+	g_task_return_boolean (task, TRUE);
 }
 
 static void
@@ -785,7 +748,6 @@ replace_file_cb (GObject      *source_object,
 	GTask *task = G_TASK (user_data);
 	GtefFileSaver *saver;
 	TaskData *task_data;
-	GFileOutputStream *file_output_stream;
 	GOutputStream *output_stream;
 	GError *error = NULL;
 
@@ -796,7 +758,8 @@ replace_file_cb (GObject      *source_object,
 	saver = g_task_get_source_object (task);
 	task_data = g_task_get_task_data (task);
 
-	file_output_stream = g_file_replace_finish (location, result, &error);
+	g_clear_object (&task_data->file_output_stream);
+	task_data->file_output_stream = g_file_replace_finish (location, result, &error);
 
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTED) &&
 	    !task_data->tried_mount)
@@ -805,6 +768,15 @@ replace_file_cb (GObject      *source_object,
 		g_error_free (error);
 		return;
 	}
+	else if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WRONG_ETAG))
+	{
+		g_task_return_new_error (task,
+					 GTEF_FILE_SAVER_ERROR,
+					 GTEF_FILE_SAVER_ERROR_EXTERNALLY_MODIFIED,
+					 _("The file is externally modified."));
+		g_error_free (error);
+		return;
+	}
 	else if (error != NULL)
 	{
 		DEBUG ({
@@ -815,7 +787,7 @@ replace_file_cb (GObject      *source_object,
 		return;
 	}
 
-	if (saver->priv->compression_type == GTK_SOURCE_COMPRESSION_TYPE_GZIP)
+	if (saver->priv->compression_type == GTEF_COMPRESSION_TYPE_GZIP)
 	{
 		GZlibCompressor *compressor;
 
@@ -825,31 +797,38 @@ replace_file_cb (GObject      *source_object,
 
 		compressor = g_zlib_compressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP, -1);
 
-		output_stream = g_converter_output_stream_new (G_OUTPUT_STREAM (file_output_stream),
+		output_stream = g_converter_output_stream_new (G_OUTPUT_STREAM (task_data->file_output_stream),
 							       G_CONVERTER (compressor));
 
 		g_object_unref (compressor);
-		g_object_unref (file_output_stream);
 	}
 	else
 	{
-		output_stream = G_OUTPUT_STREAM (file_output_stream);
+		output_stream = G_OUTPUT_STREAM (task_data->file_output_stream);
+		g_object_ref (output_stream);
 	}
 
-	/* FIXME: manage converter error? */
+	g_return_if_fail (saver->priv->encoding != NULL);
 
 	DEBUG ({
 	       g_print ("Encoding charset: %s\n",
-			gtk_source_encoding_get_charset (saver->priv->encoding));
+			gtef_encoding_get_charset (saver->priv->encoding));
 	});
 
-	if (saver->priv->encoding != gtk_source_encoding_get_utf8 ())
+	if (!gtef_encoding_is_utf8 (saver->priv->encoding))
 	{
 		GCharsetConverter *converter;
 
-		converter = g_charset_converter_new (gtk_source_encoding_get_charset (saver->priv->encoding),
+		converter = g_charset_converter_new (gtef_encoding_get_charset (saver->priv->encoding),
 						     "UTF-8",
-						     NULL);
+						     &error);
+
+		if (error != NULL)
+		{
+			g_task_return_error (task, error);
+			g_object_unref (output_stream);
+			return;
+		}
 
 		g_clear_object (&task_data->output_stream);
 		task_data->output_stream = g_converter_output_stream_new (output_stream,
@@ -878,6 +857,7 @@ begin_write (GTask *task)
 {
 	GtefFileSaver *saver;
 	gboolean create_backup;
+	const gchar *etag;
 
 	saver = g_task_get_source_object (task);
 
@@ -888,8 +868,17 @@ begin_write (GTask *task)
 	       g_print ("Make backup: %s\n", create_backup ? "yes" : "no");
 	});
 
+	if (saver->priv->flags & GTEF_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME)
+	{
+		etag = NULL;
+	}
+	else
+	{
+		etag = _gtef_file_get_etag (saver->priv->file);
+	}
+
 	g_file_replace_async (saver->priv->location,
-			      NULL,
+			      etag,
 			      create_backup,
 			      G_FILE_CREATE_NONE,
 			      g_task_get_priority (task),
@@ -898,124 +887,6 @@ begin_write (GTask *task)
 			      task);
 }
 
-static void
-check_externally_modified_cb (GObject      *source_object,
-			      GAsyncResult *result,
-			      gpointer      user_data)
-{
-	GFile *location = G_FILE (source_object);
-	GTask *task = G_TASK (user_data);
-	GtefFileSaver *saver;
-	TaskData *task_data;
-	GFileInfo *info;
-	GTimeVal old_mtime;
-	GTimeVal cur_mtime;
-	GError *error = NULL;
-
-	DEBUG ({
-	       g_print ("%s\n", G_STRFUNC);
-	});
-
-	saver = g_task_get_source_object (task);
-	task_data = g_task_get_task_data (task);
-
-	info = g_file_query_info_finish (location, result, &error);
-
-	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_MOUNTED) &&
-	    !task_data->tried_mount)
-	{
-		recover_not_mounted (task);
-		g_error_free (error);
-		return;
-	}
-
-	/* It's perfectly fine if the file doesn't exist yet. */
-	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-	{
-		g_clear_error (&error);
-	}
-	else if (error != NULL)
-	{
-		DEBUG ({
-		       g_print ("Check externally modified failed: %s\n", error->message);
-		});
-
-		g_task_return_error (task, error);
-		return;
-	}
-
-	if (_gtef_file_get_modification_time (saver->priv->file, &old_mtime) &&
-	    info != NULL &&
-	    g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
-	{
-		g_file_info_get_modification_time (info, &cur_mtime);
-
-		if (old_mtime.tv_sec != cur_mtime.tv_sec ||
-		    old_mtime.tv_usec != cur_mtime.tv_usec)
-		{
-			DEBUG ({
-			       g_print ("The file is externally modified\n");
-			});
-
-			g_task_return_new_error (task,
-						 GTEF_FILE_SAVER_ERROR,
-						 GTEF_FILE_SAVER_ERROR_EXTERNALLY_MODIFIED,
-						 _("The file is externally modified."));
-			g_object_unref (info);
-			return;
-		}
-	}
-
-	begin_write (task);
-
-	if (info != NULL)
-	{
-		g_object_unref (info);
-	}
-}
-
-static void
-check_externally_modified (GTask *task)
-{
-	GtefFileSaver *saver;
-	gboolean save_as = FALSE;
-
-	saver = g_task_get_source_object (task);
-
-	if (saver->priv->file != NULL)
-	{
-		GFile *prev_location;
-
-		prev_location = gtef_file_get_location (saver->priv->file);
-
-		/* Don't check for externally modified for a "save as" operation,
-		 * because the user has normally accepted to overwrite the file if it
-		 * already exists.
-		 */
-		save_as = (prev_location == NULL ||
-			   !g_file_equal (prev_location, saver->priv->location));
-	}
-
-	if (saver->priv->flags & GTEF_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME ||
-	    save_as)
-	{
-		begin_write (task);
-		return;
-	}
-
-	DEBUG ({
-	       g_print ("Check externally modified\n");
-	});
-
-	g_file_query_info_async (saver->priv->location,
-			         G_FILE_ATTRIBUTE_TIME_MODIFIED,
-			         G_FILE_QUERY_INFO_NONE,
-				 g_task_get_priority (task),
-				 g_task_get_cancellable (task),
-			         check_externally_modified_cb,
-			         task);
-}
-
 static void
 mount_cb (GObject      *source_object,
 	  GAsyncResult *result,
@@ -1023,12 +894,15 @@ mount_cb (GObject      *source_object,
 {
 	GFile *location = G_FILE (source_object);
 	GTask *task = G_TASK (user_data);
+	GtefFileSaver *saver;
 	GError *error = NULL;
 
 	DEBUG ({
 	       g_print ("%s\n", G_STRFUNC);
 	});
 
+	saver = g_task_get_source_object (task);
+
 	g_file_mount_enclosing_volume_finish (location, result, &error);
 
 	if (error != NULL)
@@ -1037,7 +911,12 @@ mount_cb (GObject      *source_object,
 		return;
 	}
 
-	check_externally_modified (task);
+	if (saver->priv->file != NULL)
+	{
+		_gtef_file_set_mounted (saver->priv->file);
+	}
+
+	begin_write (task);
 }
 
 static void
@@ -1119,6 +998,9 @@ gtef_file_saver_new (GtefBuffer *buffer,
  * #GtefFile:location property. If an error occurs, the previous valid
  * location is still available in #GtefFile.
  *
+ * This constructor adds %GTEF_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME to the
+ * #GtefFileSaver:flags property.
+ *
  * This constructor is suitable for a "save as" operation, or for saving a new
  * buffer for the first time.
  *
@@ -1138,6 +1020,7 @@ gtef_file_saver_new_with_target (GtefBuffer *buffer,
 			     "buffer", buffer,
 			     "file", file,
 			     "location", target_location,
+			     "flags", GTEF_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME,
 			     NULL);
 }
 
@@ -1197,22 +1080,34 @@ gtef_file_saver_get_location (GtefFileSaver *saver)
  * Since: 1.0
  */
 void
-gtef_file_saver_set_encoding (GtefFileSaver           *saver,
-			      const GtkSourceEncoding *encoding)
+gtef_file_saver_set_encoding (GtefFileSaver      *saver,
+			      const GtefEncoding *encoding)
 {
+	GtefEncoding *my_encoding;
+
 	g_return_if_fail (GTEF_IS_FILE_SAVER (saver));
 	g_return_if_fail (saver->priv->task == NULL);
 
 	if (encoding == NULL)
 	{
-		encoding = gtk_source_encoding_get_utf8 ();
+		my_encoding = gtef_encoding_new_utf8 ();
+	}
+	else
+	{
+		my_encoding = gtef_encoding_copy (encoding);
 	}
 
-	if (saver->priv->encoding != encoding)
+	if (!gtef_encoding_equals (saver->priv->encoding, my_encoding))
 	{
-		saver->priv->encoding = encoding;
+		gtef_encoding_free (saver->priv->encoding);
+		saver->priv->encoding = my_encoding;
+
 		g_object_notify (G_OBJECT (saver), "encoding");
 	}
+	else
+	{
+		gtef_encoding_free (my_encoding);
+	}
 }
 
 /**
@@ -1222,7 +1117,7 @@ gtef_file_saver_set_encoding (GtefFileSaver           *saver,
  * Returns: the encoding.
  * Since: 1.0
  */
-const GtkSourceEncoding *
+const GtefEncoding *
 gtef_file_saver_get_encoding (GtefFileSaver *saver)
 {
 	g_return_val_if_fail (GTEF_IS_FILE_SAVER (saver), NULL);
@@ -1241,8 +1136,8 @@ gtef_file_saver_get_encoding (GtefFileSaver *saver)
  * Since: 1.0
  */
 void
-gtef_file_saver_set_newline_type (GtefFileSaver        *saver,
-				  GtkSourceNewlineType  newline_type)
+gtef_file_saver_set_newline_type (GtefFileSaver   *saver,
+				  GtefNewlineType  newline_type)
 {
 	g_return_if_fail (GTEF_IS_FILE_SAVER (saver));
 	g_return_if_fail (saver->priv->task == NULL);
@@ -1261,10 +1156,10 @@ gtef_file_saver_set_newline_type (GtefFileSaver        *saver,
  * Returns: the newline type.
  * Since: 1.0
  */
-GtkSourceNewlineType
+GtefNewlineType
 gtef_file_saver_get_newline_type (GtefFileSaver *saver)
 {
-	g_return_val_if_fail (GTEF_IS_FILE_SAVER (saver), GTK_SOURCE_NEWLINE_TYPE_DEFAULT);
+	g_return_val_if_fail (GTEF_IS_FILE_SAVER (saver), GTEF_NEWLINE_TYPE_DEFAULT);
 
 	return saver->priv->newline_type;
 }
@@ -1280,8 +1175,8 @@ gtef_file_saver_get_newline_type (GtefFileSaver *saver)
  * Since: 1.0
  */
 void
-gtef_file_saver_set_compression_type (GtefFileSaver            *saver,
-				      GtkSourceCompressionType  compression_type)
+gtef_file_saver_set_compression_type (GtefFileSaver       *saver,
+				      GtefCompressionType  compression_type)
 {
 	g_return_if_fail (GTEF_IS_FILE_SAVER (saver));
 	g_return_if_fail (saver->priv->task == NULL);
@@ -1300,10 +1195,10 @@ gtef_file_saver_set_compression_type (GtefFileSaver            *saver,
  * Returns: the compression type.
  * Since: 1.0
  */
-GtkSourceCompressionType
+GtefCompressionType
 gtef_file_saver_get_compression_type (GtefFileSaver *saver)
 {
-	g_return_val_if_fail (GTEF_IS_FILE_SAVER (saver), GTK_SOURCE_COMPRESSION_TYPE_NONE);
+	g_return_val_if_fail (GTEF_IS_FILE_SAVER (saver), GTEF_COMPRESSION_TYPE_NONE);
 
 	return saver->priv->compression_type;
 }
@@ -1430,7 +1325,7 @@ gtef_file_saver_save_async (GtefFileSaver         *saver,
 								 saver->priv->newline_type,
 								 implicit_trailing_newline);
 
-	check_externally_modified (saver->priv->task);
+	begin_write (saver->priv->task);
 }
 
 /**
@@ -1467,6 +1362,7 @@ gtef_file_saver_save_finish (GtefFileSaver  *saver,
 	if (ok && saver->priv->file != NULL)
 	{
 		TaskData *task_data;
+		gchar *new_etag;
 
 		gtef_file_set_location (saver->priv->file,
 					saver->priv->location);
@@ -1485,14 +1381,9 @@ gtef_file_saver_save_finish (GtefFileSaver  *saver,
 		_gtef_file_set_readonly (saver->priv->file, FALSE);
 
 		task_data = g_task_get_task_data (G_TASK (result));
-
-		if (g_file_info_has_attribute (task_data->info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
-		{
-			GTimeVal modification_time;
-
-			g_file_info_get_modification_time (task_data->info, &modification_time);
-			_gtef_file_set_modification_time (saver->priv->file, modification_time);
-		}
+		new_etag = g_file_output_stream_get_etag (task_data->file_output_stream);
+		_gtef_file_set_etag (saver->priv->file, new_etag);
+		g_free (new_etag);
 	}
 
 	if (ok && saver->priv->source_buffer != NULL)
