@@ -1,7 +1,7 @@
 /*
  * This file is part of Gtef, a text editor library.
  *
- * Copyright 2016 - Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright 2016, 2017 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * Gtef is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the
@@ -20,6 +20,7 @@
 #include "config.h"
 #include "gtef-file.h"
 #include <glib/gi18n-lib.h>
+#include "gtef-encoding.h"
 #include "gtef-file-metadata.h"
 #include "gtef-utils.h"
 #include "gtef-enum-types.h"
@@ -54,7 +55,7 @@ struct _GtefFilePrivate
 	GtefFileMetadata *metadata;
 
 	GFile *location;
-	const GtkSourceEncoding *encoding;
+	GtefEncoding *encoding;
 	GtefNewlineType newline_type;
 	GtefCompressionType compression_type;
 
@@ -65,12 +66,10 @@ struct _GtefFilePrivate
 	gpointer mount_operation_userdata;
 	GDestroyNotify mount_operation_notify;
 
-	/* Last known modification time of 'location'. The value is updated on a
-	 * file loading or file saving.
+	/* Last known entity tag of 'location'. The value is updated on a file
+	 * loading or file saving.
 	 */
-	GTimeVal modification_time;
-
-	guint modification_time_set : 1;
+	gchar *etag;
 
 	guint externally_modified : 1;
 	guint deleted : 1;
@@ -228,7 +227,9 @@ gtef_file_finalize (GObject *object)
 {
 	GtefFilePrivate *priv = gtef_file_get_instance_private (GTEF_FILE (object));
 
+	gtef_encoding_free (priv->encoding);
 	g_free (priv->short_name);
+	g_free (priv->etag);
 
 	if (priv->untitled_number > 0)
 	{
@@ -276,7 +277,7 @@ gtef_file_class_init (GtefFileClass *klass)
 		g_param_spec_boxed ("encoding",
 				    "Encoding",
 				    "",
-				    GTK_SOURCE_TYPE_ENCODING,
+				    GTEF_TYPE_ENCODING,
 				    G_PARAM_READABLE |
 				    G_PARAM_STATIC_STRINGS);
 
@@ -513,8 +514,9 @@ gtef_file_set_location (GtefFile *file,
 	{
 		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_LOCATION]);
 
-		/* The modification_time is for the old location. */
-		priv->modification_time_set = FALSE;
+		/* The etag is for the old location. */
+		g_free (priv->etag);
+		priv->etag = NULL;
 
 		priv->externally_modified = FALSE;
 		priv->deleted = FALSE;
@@ -566,8 +568,8 @@ gtef_file_get_short_name (GtefFile *file)
 }
 
 void
-_gtef_file_set_encoding (GtefFile                *file,
-			 const GtkSourceEncoding *encoding)
+_gtef_file_set_encoding (GtefFile           *file,
+			 const GtefEncoding *encoding)
 {
 	GtefFilePrivate *priv;
 
@@ -575,9 +577,11 @@ _gtef_file_set_encoding (GtefFile                *file,
 
 	priv = gtef_file_get_instance_private (file);
 
-	if (priv->encoding != encoding)
+	if (!gtef_encoding_equals (priv->encoding, encoding))
 	{
-		priv->encoding = encoding;
+		gtef_encoding_free (priv->encoding);
+		priv->encoding = gtef_encoding_copy (encoding);
+
 		g_object_notify_by_pspec (G_OBJECT (file), properties[PROP_ENCODING]);
 	}
 }
@@ -592,7 +596,7 @@ _gtef_file_set_encoding (GtefFile                *file,
  * Returns: the character encoding.
  * Since: 1.0
  */
-const GtkSourceEncoding *
+const GtefEncoding *
 gtef_file_get_encoding (GtefFile *file)
 {
 	GtefFilePrivate *priv;
@@ -746,46 +750,39 @@ _gtef_file_set_mounted (GtefFile *file)
 	update_short_name (file);
 }
 
-gboolean
-_gtef_file_get_modification_time (GtefFile *file,
-				  GTimeVal *modification_time)
+const gchar *
+_gtef_file_get_etag (GtefFile *file)
 {
 	GtefFilePrivate *priv;
 
-	g_return_val_if_fail (modification_time != NULL, FALSE);
-
 	if (file == NULL)
 	{
-		return FALSE;
+		return NULL;
 	}
 
-	g_return_val_if_fail (GTEF_IS_FILE (file), FALSE);
+	g_return_val_if_fail (GTEF_IS_FILE (file), NULL);
 
 	priv = gtef_file_get_instance_private (file);
-
-	if (priv->modification_time_set)
-	{
-		*modification_time = priv->modification_time;
-	}
-
-	return priv->modification_time_set;
+	return priv->etag;
 }
 
 void
-_gtef_file_set_modification_time (GtefFile *file,
-				  GTimeVal  modification_time)
+_gtef_file_set_etag (GtefFile    *file,
+		     const gchar *etag)
 {
-	if (file != NULL)
+	GtefFilePrivate *priv;
+
+	if (file == NULL)
 	{
-		GtefFilePrivate *priv;
-
-		g_return_if_fail (GTEF_IS_FILE (file));
-
-		priv = gtef_file_get_instance_private (file);
-
-		priv->modification_time = modification_time;
-		priv->modification_time_set = TRUE;
+		return;
 	}
+
+	g_return_if_fail (GTEF_IS_FILE (file));
+
+	priv = gtef_file_get_instance_private (file);
+
+	g_free (priv->etag);
+	priv->etag = g_strdup (etag);
 }
 
 /**
@@ -847,7 +844,7 @@ gtef_file_check_file_on_disk (GtefFile *file)
 	}
 
 	info = g_file_query_info (priv->location,
-				  G_FILE_ATTRIBUTE_TIME_MODIFIED ","
+				  G_FILE_ATTRIBUTE_ETAG_VALUE ","
 				  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
 				  G_FILE_QUERY_INFO_NONE,
 				  NULL,
@@ -859,18 +856,16 @@ gtef_file_check_file_on_disk (GtefFile *file)
 		return;
 	}
 
-	if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED) &&
-	    priv->modification_time_set)
+	priv->deleted = FALSE;
+
+	if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ETAG_VALUE) &&
+	    priv->etag != NULL)
 	{
-		GTimeVal timeval;
+		const gchar *etag;
 
-		g_file_info_get_modification_time (info, &timeval);
+		etag = g_file_info_get_etag (info);
 
-		/* Note that the modification time can even go backwards if the
-		 * user is copying over an old file.
-		 */
-		if (timeval.tv_sec != priv->modification_time.tv_sec ||
-		    timeval.tv_usec != priv->modification_time.tv_usec)
+		if (g_strcmp0 (priv->etag, etag) != 0)
 		{
 			priv->externally_modified = TRUE;
 		}
