@@ -2,7 +2,7 @@
  * This file is part of Gtef, a text editor library.
  *
  * Copyright 2010 - Jesse van den Kieboom
- * Copyright 2014 - Sébastien Wilmet
+ * Copyright 2014, 2017 - Sébastien Wilmet
  *
  * Gtef is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the
@@ -163,12 +163,12 @@ check_mounted (SaverTestData *data)
 }
 
 static void
-test_saver (const gchar          *filename_or_uri,
-	    const gchar          *buffer_contents,
-	    const gchar          *expected_file_contents,
-	    GtkSourceNewlineType  newline_type,
-	    SavedCallback         saved_callback,
-	    gpointer              userdata)
+test_saver (const gchar     *filename_or_uri,
+	    const gchar     *buffer_contents,
+	    const gchar     *expected_file_contents,
+	    GtefNewlineType  newline_type,
+	    SavedCallback    saved_callback,
+	    gpointer         userdata)
 {
 	GFile *location;
 	GtefBuffer *buffer;
@@ -185,7 +185,7 @@ test_saver (const gchar          *filename_or_uri,
 	saver = gtef_file_saver_new_with_target (buffer, file, location);
 
 	gtef_file_saver_set_newline_type (saver, newline_type);
-	gtef_file_saver_set_encoding (saver, gtk_source_encoding_get_utf8 ());
+	gtef_file_saver_set_encoding (saver, NULL);
 
 	data = g_slice_new (SaverTestData);
 	data->saver = saver;
@@ -206,41 +206,41 @@ test_saver (const gchar          *filename_or_uri,
 
 typedef struct
 {
-	GtkSourceNewlineType type;
+	GtefNewlineType type;
 	const gchar *text;
 	const gchar *result;
 } NewLineTestData;
 
 static NewLineTestData newline_test_data[] = {
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\nhello\nworld", "\nhello\nworld\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\nhello\nworld\n", "\nhello\nworld\n\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\nhello\nworld\n\n", "\nhello\nworld\n\n\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\r\nhello\r\nworld", "\nhello\nworld\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\r\nhello\r\nworld\r\n", "\nhello\nworld\n\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\rhello\rworld", "\nhello\nworld\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\rhello\rworld\r", "\nhello\nworld\n\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\nhello\r\nworld", "\nhello\nworld\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_LF, "\nhello\r\nworld\r", "\nhello\nworld\n\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\nhello\nworld", "\nhello\nworld\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\nhello\nworld\n", "\nhello\nworld\n\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\nhello\nworld\n\n", "\nhello\nworld\n\n\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\r\nhello\r\nworld", "\nhello\nworld\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\r\nhello\r\nworld\r\n", "\nhello\nworld\n\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\rhello\rworld", "\nhello\nworld\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\rhello\rworld\r", "\nhello\nworld\n\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\nhello\r\nworld", "\nhello\nworld\n"},
+	{GTEF_NEWLINE_TYPE_LF, "\nhello\r\nworld\r", "\nhello\nworld\n\n"},
 
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\nhello\nworld", "\r\nhello\r\nworld\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\nhello\nworld\n", "\r\nhello\r\nworld\r\n\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\nhello\nworld\n\n", "\r\nhello\r\nworld\r\n\r\n\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\r\nhello\r\nworld", "\r\nhello\r\nworld\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\r\nhello\r\nworld\r\n", "\r\nhello\r\nworld\r\n\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\rhello\rworld", "\r\nhello\r\nworld\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\rhello\rworld\r", "\r\nhello\r\nworld\r\n\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\nhello\r\nworld", "\r\nhello\r\nworld\r\n"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR_LF, "\nhello\r\nworld\r", "\r\nhello\r\nworld\r\n\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\nhello\nworld", "\r\nhello\r\nworld\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\nhello\nworld\n", "\r\nhello\r\nworld\r\n\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\nhello\nworld\n\n", "\r\nhello\r\nworld\r\n\r\n\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\r\nhello\r\nworld", "\r\nhello\r\nworld\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\r\nhello\r\nworld\r\n", "\r\nhello\r\nworld\r\n\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\rhello\rworld", "\r\nhello\r\nworld\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\rhello\rworld\r", "\r\nhello\r\nworld\r\n\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\nhello\r\nworld", "\r\nhello\r\nworld\r\n"},
+	{GTEF_NEWLINE_TYPE_CR_LF, "\nhello\r\nworld\r", "\r\nhello\r\nworld\r\n\r\n"},
 
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\nhello\nworld", "\rhello\rworld\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\nhello\nworld\n", "\rhello\rworld\r\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\nhello\nworld\n\n", "\rhello\rworld\r\r\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\r\nhello\r\nworld", "\rhello\rworld\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\r\nhello\r\nworld\r\n", "\rhello\rworld\r\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\rhello\rworld", "\rhello\rworld\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\rhello\rworld\r", "\rhello\rworld\r\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\nhello\r\nworld", "\rhello\rworld\r"},
-	{GTK_SOURCE_NEWLINE_TYPE_CR, "\nhello\r\nworld\r", "\rhello\rworld\r\r"}
+	{GTEF_NEWLINE_TYPE_CR, "\nhello\nworld", "\rhello\rworld\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\nhello\nworld\n", "\rhello\rworld\r\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\nhello\nworld\n\n", "\rhello\rworld\r\r\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\r\nhello\r\nworld", "\rhello\rworld\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\r\nhello\r\nworld\r\n", "\rhello\rworld\r\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\rhello\rworld", "\rhello\rworld\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\rhello\rworld\r", "\rhello\rworld\r\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\nhello\r\nworld", "\rhello\rworld\r"},
+	{GTEF_NEWLINE_TYPE_CR, "\nhello\r\nworld\r", "\rhello\rworld\r\r"}
 };
 
 static void
@@ -285,21 +285,21 @@ test_local (void)
 	test_saver (default_local_uri,
 	            "hello world",
 		    "hello world\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 
 	test_saver (default_local_uri,
 	            "hello world\r\n",
 		    "hello world\n\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 
 	test_saver (default_local_uri,
 	            "hello world\n",
 		    "hello world\n\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 	g_free (default_local_uri);
@@ -317,21 +317,21 @@ test_remote (void)
 	test_saver (DEFAULT_REMOTE_URI,
 	            "hello world",
 		    "hello world\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 
 	test_saver (DEFAULT_REMOTE_URI,
 	            "hello world\r\n",
 		    "hello world\n\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 
 	test_saver (DEFAULT_REMOTE_URI,
 	            "hello world\n",
 		    "hello world\n\n",
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 }
@@ -409,7 +409,7 @@ test_permissions (const gchar *uri,
 	test_saver (uri,
 	            DEFAULT_CONTENT,
 		    DEFAULT_CONTENT_RESULT,
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    check_permissions_saved,
 		    GINT_TO_POINTER ((gint)permissions));
 
@@ -445,7 +445,7 @@ test_local_unowned_directory (void)
 	test_saver (unowned_local_uri,
 	            DEFAULT_CONTENT,
 		    DEFAULT_CONTENT_RESULT,
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 	g_free (unowned_local_uri);
@@ -462,7 +462,7 @@ test_remote_unowned_directory (void)
 	test_saver (unowned_remote_uri,
 	            DEFAULT_CONTENT,
 		    DEFAULT_CONTENT_RESULT,
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    NULL,
 		    NULL);
 	g_free (unowned_remote_uri);
@@ -510,7 +510,7 @@ test_unowned_group (const gchar *uri)
 	test_saver (uri,
 	            DEFAULT_CONTENT,
 		    DEFAULT_CONTENT_RESULT,
-	            GTK_SOURCE_NEWLINE_TYPE_LF,
+	            GTEF_NEWLINE_TYPE_LF,
 		    test_unowned_group_permissions,
 		    NULL);
 }
